@@ -197,6 +197,10 @@ pub fn functions() -> HashSet<&'static str>
         "arclength",
         "roll",
         "erfi",
+        "polygamma",
+        "trigamma",
+        "pochhammer",
+        "ph",
     ]
     .iter()
     .cloned()
