@@ -10,7 +10,7 @@ use crate::{
     },
     Colors, Options,
 };
-use gnuplot::{AxesCommon, Caption, Color, Figure, Fix, PointSymbol};
+use gnuplot::{Auto, AxesCommon, Caption, Color, Figure, Fix, PointSymbol, TickOption};
 use rug::Complex;
 use std::{
     io::{stdout, Write},
@@ -91,51 +91,51 @@ pub fn graph(
         }
         if d2_or_d3.0
         {
-            if lines
-            {
-                if Options::default().xr == options.xr
-                {
-                    options.xr = (
-                        points2d.iter().fold(f64::MAX, |min, x| {
-                            min.min(
-                                x[0][0]
-                                    .iter()
-                                    .chain(&x[1][0])
-                                    .fold(f64::MAX, |min, x| min.min(*x)),
-                            )
-                        }),
-                        points2d.iter().fold(f64::MIN, |max, x| {
-                            max.max(
-                                x[0][0]
-                                    .iter()
-                                    .chain(&x[1][0])
-                                    .fold(f64::MIN, |max, x| max.max(*x)),
-                            )
-                        }),
-                    )
-                }
-                if Options::default().yr == options.yr
-                {
-                    options.yr = (
-                        points2d.iter().fold(f64::MAX, |min, x| {
-                            min.min(
-                                x[0][1]
-                                    .iter()
-                                    .chain(&x[1][1])
-                                    .fold(f64::MAX, |min, x| min.min(*x)),
-                            )
-                        }),
-                        points2d.iter().fold(f64::MIN, |max, x| {
-                            max.max(
-                                x[0][1]
-                                    .iter()
-                                    .chain(&x[1][1])
-                                    .fold(f64::MIN, |max, x| max.max(*x)),
-                            )
-                        }),
-                    )
-                }
-            }
+            // if lines
+            // {
+            //     if Options::default().xr == options.xr
+            //     {
+            //         options.xr = (
+            //             points2d.iter().fold(f64::MAX, |min, x| {
+            //                 min.min(
+            //                     x[0][0]
+            //                         .iter()
+            //                         .chain(&x[1][0])
+            //                         .fold(f64::MAX, |min, x| min.min(*x)),
+            //                 )
+            //             }),
+            //             points2d.iter().fold(f64::MIN, |max, x| {
+            //                 max.max(
+            //                     x[0][0]
+            //                         .iter()
+            //                         .chain(&x[1][0])
+            //                         .fold(f64::MIN, |max, x| max.max(*x)),
+            //                 )
+            //             }),
+            //         )
+            //     }
+            //     if Options::default().yr == options.yr
+            //     {
+            //         options.yr = (
+            //             points2d.iter().fold(f64::MAX, |min, x| {
+            //                 min.min(
+            //                     x[0][1]
+            //                         .iter()
+            //                         .chain(&x[1][1])
+            //                         .fold(f64::MAX, |min, x| min.min(*x)),
+            //                 )
+            //             }),
+            //             points2d.iter().fold(f64::MIN, |max, x| {
+            //                 max.max(
+            //                     x[0][1]
+            //                         .iter()
+            //                         .chain(&x[1][1])
+            //                         .fold(f64::MIN, |max, x| max.max(*x)),
+            //                 )
+            //             }),
+            //         )
+            //     }
+            // }
             if options.vxr.0 != 0.0 || options.vxr.1 != 0.0
             {
                 options.xr = options.vxr;
@@ -144,9 +144,13 @@ pub fn graph(
             {
                 options.yr = options.vyr;
             }
-            let (xticks, yticks) = if options.ticks == -1.0
+            let (xticks, yticks) = if options.ticks == -2.0
             {
-                (Some((Fix(1.0), 1)), Some((Fix(1.0), 1)))
+                (Some((Auto, 0)), Some((Auto, 0)))
+            }
+            else if options.ticks == -1.0
+            {
+                (Some((Fix(1.0), 0)), Some((Fix(1.0), 0)))
             }
             else if options.ticks == 0.0
             {
@@ -155,26 +159,32 @@ pub fn graph(
             else
             {
                 (
-                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 1)),
-                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 1)),
+                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 0)),
+                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 0)),
                 )
             };
-            if options.scale_graph
+            let ratio = if options.scale_graph
             {
                 let (x, y) = get_terminal_dimensions_pixel();
-                let w = y as f64 / x as f64 * (options.xr.1 - options.xr.0)
-                    / (options.yr.1 - options.yr.0);
+                let rt = y as f64 / x as f64;
+                let w = rt * (options.xr.1 - options.xr.0) / (options.yr.1 - options.yr.0);
                 options.yr = (w * options.yr.0, w * options.yr.1);
+                Fix(rt)
             }
+            else
+            {
+                Auto
+            };
             if options.lines || lines
             {
                 fg.axes2d()
-                    .set_x_ticks(xticks, &[], &[])
-                    .set_y_ticks(yticks, &[], &[])
+                    .set_x_grid(true)
+                    .set_y_grid(true)
+                    .set_aspect_ratio(ratio)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(options.onaxis)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
-                    .set_x_label("x", &[])
-                    .set_y_label("y", &[])
                     .lines([0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -293,12 +303,13 @@ pub fn graph(
             else
             {
                 fg.axes2d()
-                    .set_x_ticks(xticks, &[], &[])
-                    .set_y_ticks(yticks, &[], &[])
+                    .set_x_grid(true)
+                    .set_y_grid(true)
+                    .set_aspect_ratio(ratio)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(options.onaxis)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
-                    .set_x_label("x", &[])
-                    .set_y_label("y", &[])
                     .lines([0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -417,72 +428,72 @@ pub fn graph(
         }
         if d2_or_d3.1
         {
-            if lines
-            {
-                if Options::default().xr == options.xr
-                {
-                    options.xr = (
-                        points3d.iter().fold(f64::MAX, |min, x| {
-                            min.min(
-                                x[0][0]
-                                    .iter()
-                                    .chain(&x[1][0])
-                                    .fold(f64::MAX, |min, x| min.min(*x)),
-                            )
-                        }),
-                        points3d.iter().fold(f64::MIN, |max, x| {
-                            max.max(
-                                x[0][0]
-                                    .iter()
-                                    .chain(&x[1][0])
-                                    .fold(f64::MIN, |max, x| max.max(*x)),
-                            )
-                        }),
-                    )
-                }
-                if Options::default().yr == options.yr
-                {
-                    options.yr = (
-                        points3d.iter().fold(f64::MAX, |min, x| {
-                            min.min(
-                                x[0][1]
-                                    .iter()
-                                    .chain(&x[1][1])
-                                    .fold(f64::MAX, |min, x| min.min(*x)),
-                            )
-                        }),
-                        points3d.iter().fold(f64::MIN, |max, x| {
-                            max.max(
-                                x[0][1]
-                                    .iter()
-                                    .chain(&x[1][1])
-                                    .fold(f64::MIN, |max, x| max.max(*x)),
-                            )
-                        }),
-                    )
-                }
-                if Options::default().zr == options.zr
-                {
-                    options.zr = (
-                        points3d.iter().fold(f64::MAX, |min, x| {
-                            min.min(
-                                x[0][2]
-                                    .iter()
-                                    .chain(&x[1][2])
-                                    .fold(f64::MAX, |min, x| min.min(*x)),
-                            )
-                        }),
-                        points3d.iter().fold(f64::MIN, |max, x| {
-                            max.max(
-                                x[0][2]
-                                    .iter()
-                                    .chain(&x[1][2])
-                                    .fold(f64::MIN, |max, x| max.max(*x)),
-                            )
-                        }),
-                    )
-                }
-            }
+            // if lines
+            // {
+            //     if Options::default().xr == options.xr
+            //     {
+            //         options.xr = (
+            //             points3d.iter().fold(f64::MAX, |min, x| {
+            //                 min.min(
+            //                     x[0][0]
+            //                         .iter()
+            //                         .chain(&x[1][0])
+            //                         .fold(f64::MAX, |min, x| min.min(*x)),
+            //                 )
+            //             }),
+            //             points3d.iter().fold(f64::MIN, |max, x| {
+            //                 max.max(
+            //                     x[0][0]
+            //                         .iter()
+            //                         .chain(&x[1][0])
+            //                         .fold(f64::MIN, |max, x| max.max(*x)),
+            //                 )
+            //             }),
+            //         )
+            //     }
+            //     if Options::default().yr == options.yr
+            //     {
+            //         options.yr = (
+            //             points3d.iter().fold(f64::MAX, |min, x| {
+            //                 min.min(
+            //                     x[0][1]
+            //                         .iter()
+            //                         .chain(&x[1][1])
+            //                         .fold(f64::MAX, |min, x| min.min(*x)),
+            //                 )
+            //             }),
+            //             points3d.iter().fold(f64::MIN, |max, x| {
+            //                 max.max(
+            //                     x[0][1]
+            //                         .iter()
+            //                         .chain(&x[1][1])
+            //                         .fold(f64::MIN, |max, x| max.max(*x)),
+            //                 )
+            //             }),
+            //         )
+            //     }
+            //     if Options::default().zr == options.zr
+            //     {
+            //         options.zr = (
+            //             points3d.iter().fold(f64::MAX, |min, x| {
+            //                 min.min(
+            //                     x[0][2]
+            //                         .iter()
+            //                         .chain(&x[1][2])
+            //                         .fold(f64::MAX, |min, x| min.min(*x)),
+            //                 )
+            //             }),
+            //             points3d.iter().fold(f64::MIN, |max, x| {
+            //                 max.max(
+            //                     x[0][2]
+            //                         .iter()
+            //                         .chain(&x[1][2])
+            //                         .fold(f64::MIN, |max, x| max.max(*x)),
+            //                 )
+            //             }),
+            //         )
+            //     }
+            // }
             if options.vxr.0 != 0.0 || options.vxr.1 != 0.0
             {
                 options.xr = options.vxr;
@@ -495,12 +506,16 @@ pub fn graph(
             {
                 options.zr = options.vzr;
             }
-            let (xticks, yticks, zticks) = if options.ticks == -1.0
+            let (xticks, yticks, zticks) = if options.ticks == -2.0
+            {
+                (Some((Auto, 0)), Some((Auto, 0)), Some((Auto, 0)))
+            }
+            else if options.ticks == -1.0
             {
                 (
-                    Some((Fix(1.0), 1)),
-                    Some((Fix(1.0), 1)),
-                    Some((Fix(1.0), 1)),
+                    Some((Fix(1.0), 0)),
+                    Some((Fix(1.0), 0)),
+                    Some((Fix(1.0), 0)),
                 )
             }
             else if options.ticks == 0.0
@@ -510,9 +525,9 @@ pub fn graph(
             else
             {
                 (
-                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 1)),
-                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 1)),
-                    Some((Fix((options.zr.1 - options.zr.0) / options.ticks), 1)),
+                    Some((Fix((options.xr.1 - options.xr.0) / options.ticks), 0)),
+                    Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 0)),
+                    Some((Fix((options.zr.1 - options.zr.0) / options.ticks), 0)),
                 )
             };
             if options.surface
@@ -522,15 +537,18 @@ pub fn graph(
                 && (points3d[0][0][2].is_empty() || points3d[0][1][2].is_empty())
             {
                 fg.axes3d()
-                    .set_x_ticks(xticks, &[], &[])
-                    .set_y_ticks(yticks, &[], &[])
-                    .set_z_ticks(zticks, &[], &[])
+                    .set_x_grid(true)
+                    .set_y_grid(true)
+                    .set_z_grid(true)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
                     .set_x_label("x", &[])
-                    .set_y_label("y", &[])
-                    .set_z_label("z", &[])
+                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
+                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .surface(
@@ -551,15 +569,18 @@ pub fn graph(
             else if options.lines || lines
             {
                 fg.axes3d()
-                    .set_x_ticks(xticks, &[], &[])
-                    .set_y_ticks(yticks, &[], &[])
-                    .set_z_ticks(zticks, &[], &[])
+                    .set_x_grid(true)
+                    .set_y_grid(true)
+                    .set_z_grid(true)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
                     .set_x_label("x", &[])
-                    .set_y_label("y", &[])
-                    .set_z_label("z", &[])
+                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
+                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -732,15 +753,18 @@ pub fn graph(
             else
             {
                 fg.axes3d()
-                    .set_x_ticks(xticks, &[], &[])
-                    .set_y_ticks(yticks, &[], &[])
-                    .set_z_ticks(zticks, &[], &[])
+                    .set_x_grid(true)
+                    .set_y_grid(true)
+                    .set_z_grid(true)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
                     .set_x_label("x", &[])
-                    .set_y_label("y", &[])
-                    .set_z_label("z", &[])
+                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
+                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -955,6 +979,7 @@ pub fn get_list_2d(
         Vec::with_capacity(func.2.samples_2d + 1),
         Vec::with_capacity(func.2.samples_2d + 1),
     ];
+    let mut nan = true;
     let den_range = (func.2.xr.1 - func.2.xr.0) / func.2.samples_2d as f64;
     let mut zero = (false, false);
     let list = func.0.iter().any(|c| {
@@ -990,13 +1015,14 @@ pub fn get_list_2d(
         {
             Ok(Num(num)) =>
             {
-                if num.real().is_nan()
+                if num.real().is_nan() || num.imag().is_nan()
                 {
                     continue;
                 }
-                let complex = num.real().is_finite();
+                let complex = !num.real().is_infinite();
                 if complex
                 {
+                    nan = false;
                     let f = num.real().to_f64();
                     if (f * 1e8).round() / 1e8 != 0.0
                     {
@@ -1005,8 +1031,9 @@ pub fn get_list_2d(
                     data[0][0].push(n);
                     data[0][1].push(f);
                 }
-                if num.imag().is_finite()
+                if !num.imag().is_infinite()
                 {
+                    nan = false;
                     let f = num.imag().to_f64();
                     if (f * 1e8).round() / 1e8 != 0.0
                     {
@@ -1019,6 +1046,15 @@ pub fn get_list_2d(
                     }
                     data[1][1].push(f);
                 }
+                else
+                {
+                    if !complex
+                    {
+                        data[0][0].push(n);
+                        data[0][1].push(f64::NAN);
+                    }
+                    data[1][1].push(f64::NAN);
+                }
             }
             Ok(Vector(v)) =>
             {
@@ -1026,13 +1062,14 @@ pub fn get_list_2d(
                 {
                     for num in v
                     {
-                        if num.real().is_nan()
+                        if num.real().is_nan() || num.imag().is_nan()
                         {
                             continue;
                         }
-                        let complex = num.real().is_finite();
+                        let complex = !num.real().is_infinite();
                         if complex
                         {
+                            nan = false;
                             let f = num.real().to_f64();
                             if (f * 1e8).round() / 1e8 != 0.0
                             {
@@ -1041,8 +1078,9 @@ pub fn get_list_2d(
                             data[0][0].push(n);
                             data[0][1].push(f);
                         }
-                        if num.imag().is_finite()
+                        if !num.imag().is_infinite()
                         {
+                            nan = false;
                             let f = num.imag().to_f64();
                             if (f * 1e8).round() / 1e8 != 0.0
                             {
@@ -1055,10 +1093,20 @@ pub fn get_list_2d(
                             }
                             data[1][1].push(f);
                         }
+                        else
+                        {
+                            if !complex
+                            {
+                                data[0][0].push(n);
+                                data[0][1].push(f64::NAN);
+                            }
+                            data[1][1].push(f64::NAN);
+                        }
                     }
                 }
                 else if v.len() == 3
                 {
+                    nan = false;
                     let xr = v[0].real().to_f64();
                     let yr = v[1].real().to_f64();
                     let zr = v[2].real().to_f64();
@@ -1086,6 +1134,7 @@ pub fn get_list_2d(
                 }
                 else if v.len() == 2
                 {
+                    nan = false;
                     let xr = v[0].real().to_f64();
                     let yr = v[1].real().to_f64();
                     let xi = v[0].imag().to_f64();
@@ -1110,13 +1159,14 @@ pub fn get_list_2d(
                 {
                     for num in v
                     {
-                        if num.real().is_nan()
+                        if num.real().is_nan() || num.imag().is_nan()
                         {
                             continue;
                         }
-                        let complex = num.real().is_finite();
+                        let complex = !num.real().is_infinite();
                         if complex
                         {
+                            nan = false;
                             let f = num.real().to_f64();
                             if (f * 1e8).round() / 1e8 != 0.0
                             {
@@ -1125,8 +1175,9 @@ pub fn get_list_2d(
                             data[0][0].push(n);
                             data[0][1].push(f);
                         }
-                        if num.imag().is_finite()
+                        if !num.imag().is_infinite()
                         {
+                            nan = false;
                             let f = num.imag().to_f64();
                             if (f * 1e8).round() / 1e8 != 0.0
                             {
@@ -1139,13 +1190,22 @@ pub fn get_list_2d(
                             }
                             data[1][1].push(f);
                         }
+                        else
+                        {
+                            if !complex
+                            {
+                                data[0][0].push(n);
+                                data[0][1].push(f64::NAN);
+                            }
+                            data[1][1].push(f64::NAN);
+                        }
                     }
                 }
             }
             Err(s) =>
             {
                 println!("{}", s);
-                return (Default::default(), Default::default(), (false, false));
+                return Default::default();
             }
             _ =>
             {}
@@ -1166,6 +1226,15 @@ pub fn get_list_2d(
         data[1][1] = Vec::new();
         data3d[1] = Vec::new();
     }
+    if (data[0][0].is_empty()
+        && data[1][0].is_empty()
+        && data3d[0].is_empty()
+        && data3d[1].is_empty())
+        || nan
+    {
+        println!("\x1b[G\ngraph is all 0, nan or infinity");
+        return Default::default();
+    }
     (data, data3d, zero)
 }
 #[allow(clippy::type_complexity)]
@@ -1207,6 +1276,7 @@ pub fn get_list_3d(
     let mut modified: Vec<NumStr>;
     let mut modifiedvars: Vec<(String, Vec<NumStr>)>;
     let mut zero = (false, false);
+    let mut nan = true;
     let list = func.0.iter().any(|c| {
         if let Str(s) = c
         {
@@ -1247,24 +1317,28 @@ pub fn get_list_3d(
             {
                 Ok(Num(num)) =>
                 {
-                    if num.real().is_nan()
+                    if num.real().is_nan() || num.imag().is_nan()
                     {
                         continue;
                     }
-                    let complex = num.real().is_finite();
+                    let complex = !num.real().is_infinite();
                     if complex
                     {
-                        if (num.real().to_f64() * 1e8).round() / 1e8 != 0.0
+                        nan = false;
+                        let g = num.real().to_f64();
+                        if (g * 1e8).round() / 1e8 != 0.0
                         {
                             zero.0 = true
                         }
                         data[0][0].push(n);
                         data[0][1].push(f);
-                        data[0][2].push(num.real().to_f64());
+                        data[0][2].push(g);
                     }
-                    if num.imag().is_finite()
+                    if !num.imag().is_infinite()
                     {
-                        if (num.imag().to_f64() * 1e8).round() / 1e8 != 0.0
+                        nan = false;
+                        let g = num.imag().to_f64();
+                        if (g * 1e8).round() / 1e8 != 0.0
                         {
                             zero.1 = true
                         }
@@ -1274,7 +1348,17 @@ pub fn get_list_3d(
                             data[0][1].push(f);
                             data[0][2].push(f64::NAN);
                         }
-                        data[1][2].push(num.imag().to_f64());
+                        data[1][2].push(g);
+                    }
+                    else
+                    {
+                        if !complex
+                        {
+                            data[0][0].push(n);
+                            data[0][1].push(f);
+                            data[0][2].push(f64::NAN);
+                        }
+                        data[1][2].push(f64::NAN);
                     }
                 }
                 Ok(Vector(v)) =>
@@ -1283,24 +1367,28 @@ pub fn get_list_3d(
                     {
                         for num in v
                         {
-                            if num.real().is_nan()
+                            if num.real().is_nan() || num.imag().is_nan()
                             {
                                 continue;
                             }
-                            let complex = num.real().is_finite();
+                            let complex = !num.real().is_infinite();
                             if complex
                             {
-                                if (num.real().to_f64() * 1e8).round() / 1e8 != 0.0
+                                nan = false;
+                                let g = num.real().to_f64();
+                                if (g * 1e8).round() / 1e8 != 0.0
                                 {
                                     zero.0 = true
                                 }
                                 data[0][0].push(n);
                                 data[0][1].push(f);
-                                data[0][2].push(num.real().to_f64());
+                                data[0][2].push(g);
                             }
-                            if num.imag().is_finite()
+                            if !num.imag().is_infinite()
                             {
-                                if (num.imag().to_f64() * 1e8).round() / 1e8 != 0.0
+                                nan = false;
+                                let g = num.imag().to_f64();
+                                if (g * 1e8).round() / 1e8 != 0.0
                                 {
                                     zero.1 = true
                                 }
@@ -1310,12 +1398,23 @@ pub fn get_list_3d(
                                     data[0][1].push(f);
                                     data[0][2].push(f64::NAN);
                                 }
-                                data[1][2].push(num.imag().to_f64());
+                                data[1][2].push(g);
+                            }
+                            else
+                            {
+                                if !complex
+                                {
+                                    data[0][0].push(n);
+                                    data[0][1].push(f);
+                                    data[0][2].push(f64::NAN);
+                                }
+                                data[1][2].push(f64::NAN);
                             }
                         }
                     }
                     else if v.len() == 3
                     {
+                        nan = false;
                         let xr = v[0].real().to_f64();
                         let yr = v[1].real().to_f64();
                         let zr = v[2].real().to_f64();
@@ -1343,6 +1442,7 @@ pub fn get_list_3d(
                     }
                     else if v.len() == 2
                     {
+                        nan = false;
                         d2 = true;
                         let xr = v[0].real().to_f64();
                         let yr = v[1].real().to_f64();
@@ -1368,24 +1468,28 @@ pub fn get_list_3d(
                     {
                         for num in v
                         {
-                            if num.real().is_nan()
+                            if num.real().is_nan() || num.imag().is_nan()
                             {
                                 continue;
                             }
-                            let complex = num.real().is_finite();
+                            let complex = !num.real().is_infinite();
                             if complex
                             {
-                                if (num.real().to_f64() * 1e8).round() / 1e8 != 0.0
+                                nan = false;
+                                let g = num.real().to_f64();
+                                if (g * 1e8).round() / 1e8 != 0.0
                                 {
                                     zero.0 = true
                                 }
                                 data[0][0].push(n);
                                 data[0][1].push(f);
-                                data[0][2].push(num.real().to_f64());
+                                data[0][2].push(g);
                             }
-                            if num.imag().is_finite()
+                            if !num.imag().is_infinite()
                             {
-                                if (num.imag().to_f64() * 1e8).round() / 1e8 != 0.0
+                                nan = false;
+                                let g = num.imag().to_f64();
+                                if (g * 1e8).round() / 1e8 != 0.0
                                 {
                                     zero.1 = true
                                 }
@@ -1395,7 +1499,17 @@ pub fn get_list_3d(
                                     data[0][1].push(f);
                                     data[0][2].push(f64::NAN);
                                 }
-                                data[1][2].push(num.imag().to_f64());
+                                data[1][2].push(g);
+                            }
+                            else
+                            {
+                                if !complex
+                                {
+                                    data[0][0].push(n);
+                                    data[0][1].push(f);
+                                    data[0][2].push(f64::NAN);
+                                }
+                                data[1][2].push(f64::NAN);
                             }
                         }
                     }
@@ -1403,7 +1517,7 @@ pub fn get_list_3d(
                 Err(s) =>
                 {
                     println!("{}", s);
-                    return (Default::default(), Default::default(), false);
+                    return Default::default();
                 }
                 _ =>
                 {}
@@ -1422,6 +1536,11 @@ pub fn get_list_3d(
     {
         data[1][2] = Vec::new();
     }
+    if (data[0][0].is_empty() && data[1][0].is_empty()) || nan
+    {
+        println!("\x1b[G\ngraph is all 0, nan or infinity");
+        return Default::default();
+    }
     (data, zero, d2)
 }
 fn fail(options: Options, colors: &Colors, input: String)
@@ -1486,7 +1605,7 @@ fn get_data(
                     let re = n.real().to_f64();
                     for i in 0..func.2.samples_2d
                     {
-                        if re != 0.0
+                        if re != 0.0 || im == 0.0
                         {
                             points2d[0][0].push(func.2.xr.0 + change * i as f64);
                             points2d[0][1].push(re);
@@ -1709,12 +1828,12 @@ fn get_data(
             if !data3d[0].is_empty() || !data3d[1].is_empty()
             {
                 d2_or_d3 = (false, true);
-                points3d[0][0] = points2d[0][0].clone();
-                points3d[0][1] = points2d[0][1].clone();
-                points3d[0][2] = data3d[0].clone();
-                points3d[1][0] = points2d[1][0].clone();
-                points3d[1][1] = points2d[1][1].clone();
-                points3d[1][2] = data3d[1].clone();
+                points3d[0][0].clone_from(&points2d[0][0]);
+                points3d[0][1].clone_from(&points2d[0][1]);
+                points3d[0][2].clone_from(&data3d[0]);
+                points3d[1][0].clone_from(&points2d[1][0]);
+                points3d[1][1].clone_from(&points2d[1][1]);
+                points3d[1][2].clone_from(&data3d[1]);
                 points2d = Default::default();
             }
             else if points2d[0][1].is_empty() && points2d[1][1].is_empty()
@@ -1749,9 +1868,9 @@ fn get_data(
             {
                 re_or_im.1 = false;
                 d2_or_d3 = (false, true);
-                points3d[0][0] = points2d[0][0].clone();
-                points3d[0][1] = points2d[0][1].clone();
-                points3d[0][2] = points2d[1][1].clone();
+                points3d[0][0].clone_from(&points2d[0][0]);
+                points3d[0][1].clone_from(&points2d[0][1]);
+                points3d[0][2].clone_from(&points2d[1][1]);
                 points2d = Default::default();
             }
         }
@@ -1763,10 +1882,10 @@ fn get_data(
             if d2
             {
                 d2_or_d3 = (true, false);
-                points2d[0][0] = points3d[0][0].clone();
-                points2d[0][1] = points3d[0][1].clone();
-                points2d[1][0] = points3d[1][0].clone();
-                points2d[1][1] = points3d[1][1].clone();
+                points2d[0][0].clone_from(&points3d[0][0]);
+                points2d[0][1].clone_from(&points3d[0][1]);
+                points2d[1][0].clone_from(&points3d[1][0]);
+                points2d[1][1].clone_from(&points3d[1][1]);
                 points3d = Default::default();
             }
             else if points3d[0][2].is_empty() && points3d[1][2].is_empty()
