@@ -20,7 +20,8 @@ pub fn help()
     --point [char] point style for graphing\x1b[G\n\
     --sci toggles scientific notation\x1b[G\n\
     --base=[num] sets the number base (2,8,16)\x1b[G\n\
-    --ticks=[num] sets amount of ticks, -1 will be at every whole number, 0 will be none\x1b[G\n\
+    --ticks=[num] sets amount of ticks, -2 will be auto, -1 will be at every whole number, 0 will be none\x1b[G\n\
+    --onaxis toggles showing the ticks on the x/y/z axis\x1b[G\n\
     --prompt toggles the prompt\x1b[G\n\
     --color toggles color output, toggled by default when running from arguments\x1b[G\n\
     --comma toggles comma seperation\x1b[G\n\
@@ -39,6 +40,7 @@ pub fn help()
     --multi toggles multi line display for matrixes\x1b[G\n\
     --tabbed toggles tabbed display for matrixes\x1b[G\n\
     --depth display 2d complex graphs in 3d with imag #'s going up/down on the z axis\x1b[G\n\
+    --surface displays a colored surface(based on z value) for 3d graphing, only supports 1 graph\x1b[G\n\
     --flat display 2d complex graphs like they are on the 2d number line\x1b[G\n\
     --small_e use small e notation, like 5e2=5*10^2, instead of capital 'E' for scientific notation. only works with a number before and number or '-' sign after the 'e' otherwise assumes euler number\x1b[G\n\
     --scalegraph scales the y part of a 2d graph to the users screen size\x1b[G\n\x1b[G\n\
@@ -84,7 +86,7 @@ pub fn help()
     - ceil, floor, round, int, frac\x1b[G\n\
     - fact, doublefact, subfact\x1b[G\n\
     - sinc, cis, exp\x1b[G\n\
-    - zeta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C, P(n,r)\x1b[G\n\
+    - zeta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
     - deg, rad, grad\x1b[G\n\
     - re, im, split(x+yi={{x,y}})\x1b[G\n\
     - factors, prime, isprime, gcd, lcm\x1b[G\n\
@@ -98,7 +100,7 @@ pub fn help()
     - betaP(α,β,x) (beta distribution pdf) I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\x1b[G\n\
     - roll{{a,b,c...}} rolls die dice{{a,b,c...}} gets the frequency data any amount of different sided die, where a/b/c are number of faces for each die, both also accept {{{{first_dice_face,# of die}},{{second_dice_face,# of die}}...}}\x1b[G\n\
     - lim(x,f(x),point (,side)) both sides are checked by default, -1 for left, 1 for right\x1b[G\n\
-    - slope(x,f(x),point), can add a 0 to the args to not combine the x and y slopes for parametric equations, and for area\x1b[G\n\
+    - slope(x,f(x),point (,nth derivitive)), can add a 0 to the args to not combine the x and y slopes for parametric equations, and for area\x1b[G\n\
     - area(x,f(x),from,to (,amount of data points) (,0) ), length(x,f(x),from,to (,amount of data points) ) (bracketed means optional)\n\x1b[G\n\
     Vector operations/functions:\x1b[G\n\
     - dot({{vec1}},{{vec2}}), cross({{vec1}},{{vec2}}), proj/project({{vec1}},{{vec2}})\x1b[G\n\
