@@ -16,7 +16,9 @@ use crate::{
     graph::graph,
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
-    misc::{clearln, convert, get_terminal_dimensions, prompt, read_single_char, write},
+    misc::{
+        clearln, convert, get_terminal_dimensions, handle_err, prompt, read_single_char, write,
+    },
     options::{arg_opts, commands, file_opts, silent_commands},
     parse::input_var,
     print::{print_answer, print_concurrent},
@@ -34,12 +36,10 @@ use std::{
 //surface area of a 3d curve
 //rpn
 //matrix exponentiation
+//do as much as you can before graphing, make '=' at end show unsimplified, and no = show simplififed
+//stop vectors or matrixes disipearing graphically
+//maybe nth area
 //support units properly, add a part to the Num struct where it just stores the unit which then can be dealt with in complex or smth
-//maybe assume brackets for say log(10x^(2,4) to log(10x^(2),4) if possible
-//dont use strings in math.rs
-//dont clear terminal first, write the line, \x1b[K, then on the last line \x1[b[J
-//remove \x1b[A repeats and \x08 and said repeats \x1b[D
-//do as much as you can before graphing
 #[derive(Clone)]
 pub struct Variable
 {
@@ -115,6 +115,7 @@ pub struct Options
     deg: AngleType,
     base: usize,
     ticks: f64,
+    onaxis: bool,
     tau: bool,
     polar: bool,
     frac: bool,
@@ -159,6 +160,7 @@ impl Default for Options
             deg: Radians,
             base: 10,
             ticks: 20.0,
+            onaxis: true,
             tau: false,
             polar: false,
             frac: true,
@@ -228,7 +230,7 @@ fn main()
             {
                 if let Ok(line) = line
                 {
-                    if !line.starts_with('#')
+                    if !line.starts_with('#') && !line.is_empty()
                     {
                         args.push(line);
                     }
@@ -482,13 +484,16 @@ fn main()
                         }
                     }
                 }
-                if let Some(time) = watch
+                if !graphable && !varcheck
                 {
-                    println!(" {}", time.elapsed().as_nanos());
-                }
-                else if !graphable && !varcheck
-                {
-                    println!();
+                    if let Some(time) = watch
+                    {
+                        println!(" {}", time.elapsed().as_nanos());
+                    }
+                    else
+                    {
+                        println!();
+                    }
                 }
             }
         }
@@ -519,6 +524,7 @@ fn main()
             let mut start = 0;
             let mut end = 0;
             let mut slow = false;
+            let mut firstslow = false;
             loop
             {
                 let c = read_single_char();
@@ -548,6 +554,7 @@ fn main()
                                 end,
                                 c == '\n',
                             );
+                            slow = watch.elapsed().as_millis() > options.slowcheck;
                         }
                         if !input.is_empty()
                         {
@@ -597,7 +604,7 @@ fn main()
                         }
                         if i == lines.len()
                         {
-                            current = input.clone();
+                            current.clone_from(&input);
                         }
                         else
                         {
@@ -617,13 +624,22 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
@@ -632,14 +648,14 @@ fn main()
                         {
                             let time = watch.elapsed().as_nanos();
                             print!(
-                                " {}{}",
+                                " {}\x1b[{}D",
                                 time,
-                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
+                                time.to_string().len() + 1 + end - placement
                             );
                         }
-                        else
+                        else if end - placement != 0
                         {
-                            print!("{}", "\x08".repeat(end - placement));
+                            print!("\x1b[{}D", end - placement)
                         }
                     }
                     '\x7F' =>
@@ -661,7 +677,7 @@ fn main()
                         }
                         if i == lines.len()
                         {
-                            current = input.clone();
+                            current.clone_from(&input);
                         }
                         else
                         {
@@ -681,13 +697,22 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
@@ -696,14 +721,14 @@ fn main()
                         {
                             let time = watch.elapsed().as_nanos();
                             print!(
-                                " {}{}",
+                                " {}\x1b[{}D",
                                 time,
-                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
+                                time.to_string().len() + 1 + end - placement
                             );
                         }
-                        else
+                        else if end - placement != 0
                         {
-                            print!("{}", "\x08".repeat(end - placement));
+                            print!("\x1b[{}D", end - placement)
                         }
                     }
                     '\x11' =>
@@ -735,13 +760,22 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
@@ -767,18 +801,30 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - placement));
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
                     }
                     '\x19' =>
                     {
@@ -799,18 +845,30 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - placement));
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
                     }
                     '\x17' =>
                     {
@@ -833,18 +891,30 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - placement));
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
                     }
                     '\x16' =>
                     {
@@ -867,18 +937,30 @@ fn main()
                                 );
                                 if watch.elapsed().as_millis() > options.slowcheck
                                 {
+                                    firstslow = true;
                                     slow = true;
-                                    print!(
-                                            "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                            input.len() + if options.prompt { 2 } else { 0 }
-                                        );
                                 }
                             }
+                            else if firstslow
+                            {
+                                firstslow = false;
+                                handle_err(
+                                    "too slow, will print on enter",
+                                    &input,
+                                    options,
+                                    &colors,
+                                    start,
+                                    end,
+                                )
+                            }
                             else
                             {
                                 clearln(&input, start, end, options, &colors);
                             }
-                            print!("{}", "\x08".repeat(end - placement));
+                            if end - placement != 0
+                            {
+                                print!("\x1b[{}D", end - placement)
+                            }
                         }
                     }
                     '\x15' =>
@@ -898,18 +980,30 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - placement));
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
                     }
                     '\x10' =>
                     {
@@ -939,18 +1033,30 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
                         }
-                        print!("{}", "\x08".repeat(end - placement));
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
                     }
                     '\x1D' | '\x05' =>
                     {
@@ -961,8 +1067,27 @@ fn main()
                             continue;
                         }
                         input = lines[i].clone().chars().collect::<Vec<char>>();
-                        placement = input.len();
-                        end = input.len();
+                        slow = input.starts_with(&['\0']);
+                        if slow
+                        {
+                            input.remove(0);
+                            firstslow = false;
+                            placement = input.len();
+                            end = input.len();
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            placement = input.len();
+                            end = input.len();
+                        }
                         start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
@@ -973,7 +1098,7 @@ fn main()
                             input.len()
                                 - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
-                        if options.real_time_output
+                        if options.real_time_output && !slow
                         {
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
@@ -986,13 +1111,7 @@ fn main()
                                 false,
                             );
                             slow = watch.elapsed().as_millis() > options.slowcheck;
-                            if slow
-                            {
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
-                            }
+                            firstslow = slow
                         }
                         else
                         {
@@ -1006,14 +1125,33 @@ fn main()
                         if i >= lines.len()
                         {
                             i = lines.len();
-                            input = current.clone();
+                            input.clone_from(&current);
                         }
                         else
                         {
                             input = lines[i].clone().chars().collect::<Vec<char>>();
                         }
-                        placement = input.len();
-                        end = input.len();
+                        slow = input.starts_with(&['\0']);
+                        if slow
+                        {
+                            input.remove(0);
+                            firstslow = false;
+                            placement = input.len();
+                            end = input.len();
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            placement = input.len();
+                            end = input.len();
+                        }
                         start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
@@ -1024,7 +1162,7 @@ fn main()
                             input.len()
                                 - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
-                        if options.real_time_output
+                        if options.real_time_output && !slow
                         {
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
@@ -1037,13 +1175,7 @@ fn main()
                                 false,
                             );
                             slow = watch.elapsed().as_millis() > options.slowcheck;
-                            if slow
-                            {
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
-                            }
+                            firstslow = slow;
                         }
                         else
                         {
@@ -1064,7 +1196,7 @@ fn main()
                                 end = input.len()
                             }
                             clearln(&input, start, end, options, &colors);
-                            print!("{}", "\x08".repeat(end - placement))
+                            print!("\x1b[{}D", end - placement)
                         }
                         else if placement != 0
                         {
@@ -1121,7 +1253,10 @@ fn main()
                                 end -= start - placement;
                                 start = start - (start - placement);
                                 clearln(&input, start, end, options, &colors);
-                                print!("{}", "\x08".repeat(end - placement));
+                                if end - placement != 0
+                                {
+                                    print!("\x1b[{}D", end - placement)
+                                }
                             }
                             else if placement == s
                             {
@@ -1196,7 +1331,7 @@ fn main()
                         }
                         if i == lines.len()
                         {
-                            current = input.clone();
+                            current.clone_from(&input);
                         }
                         else
                         {
@@ -1216,13 +1351,22 @@ fn main()
                             );
                             if watch.elapsed().as_millis() > options.slowcheck
                             {
+                                firstslow = true;
                                 slow = true;
-                                print!(
-                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
-                                        input.len() + if options.prompt { 2 } else { 0 }
-                                    );
                             }
                         }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
                         else
                         {
                             clearln(&input, start, end, options, &colors);
@@ -1231,14 +1375,14 @@ fn main()
                         {
                             let time = watch.elapsed().as_nanos();
                             print!(
-                                " {}{}",
+                                " {}\x1b[{}D",
                                 time,
-                                "\x08".repeat(time.to_string().len() + 1 + end - placement)
+                                time.to_string().len() + 1 + end - placement
                             );
                         }
-                        else if placement != input.len()
+                        else if end - placement != 0
                         {
-                            print!("{}", "\x08".repeat(end - placement));
+                            print!("\x1b[{}D", end - placement)
                         }
                     }
                 }
@@ -1270,7 +1414,7 @@ fn main()
                 continue;
             }
             write(
-                &input
+                input
                     .iter()
                     .collect::<String>()
                     .replace("small_e", "smalle")
@@ -1280,6 +1424,7 @@ fn main()
                     .replace("fraciter", "frac_iter"),
                 file.as_mut().unwrap(),
                 unmod_lines.as_mut().unwrap(),
+                slow,
             );
         }
         if varcheck
@@ -1330,6 +1475,7 @@ fn main()
                         continue;
                     }
                     {
+                        options.prec = options.graph_prec;
                         let split = s.split(|c| c == ';');
                         let count = split.clone().count();
                         if count != 1
@@ -1357,7 +1503,7 @@ fn main()
                                     )
                                     {
                                         print!(
-                                            "\x1b[A\x1b[G\x1b[K{}\n\x1b[G{}",
+                                            "\x1b[G\x1b[A\x1b[K{}\n\x1b[G{}",
                                             s,
                                             prompt(options, &colors)
                                         );
@@ -1367,6 +1513,7 @@ fn main()
                             }
                         }
                     }
+                    options.prec = options.graph_prec;
                     funcs.push(
                         match input_var(
                             &inputs[i],
