@@ -238,18 +238,22 @@ pub fn no_col(input: &str, color: bool) -> Vec<char>
         input.chars().collect::<Vec<char>>()
     }
 }
-pub fn write(input: &str, file: &mut File, lines: &mut Vec<String>)
+pub fn write(mut input: String, file: &mut File, lines: &mut Vec<String>, slow: bool)
 {
-    if !lines.is_empty() && lines.last().unwrap() != input && !input.replace(' ', "").is_empty()
+    if !lines.is_empty() && lines.last().unwrap() != &input && !input.replace(' ', "").is_empty()
     {
-        lines.push(input.to_string());
+        if slow
+        {
+            input.insert(0, '\0');
+        }
+        lines.push(input.clone());
         writeln!(file, "{}", input).unwrap();
     }
 }
 pub fn clearln(input: &[char], start: usize, end: usize, options: Options, colors: &Colors)
 {
     print!(
-        "\x1b[G\x1b[K{}{}{}",
+        "\x1b[G{}{}\x1b[K{}",
         prompt(options, colors),
         to_output(&input[start..end], options.color, colors),
         if options.color { "\x1b[0m" } else { "" }
@@ -258,7 +262,7 @@ pub fn clearln(input: &[char], start: usize, end: usize, options: Options, color
 pub fn clear(input: &[char], start: usize, end: usize, options: Options, colors: &Colors)
 {
     print!(
-        "\x1b[G\x1b[J{}{}{}",
+        "\x1b[G{}{}\x1b[J{}",
         prompt(options, colors),
         to_output(&input[start..end], options.color, colors),
         if options.color { "\x1b[0m" } else { "" }
@@ -364,10 +368,18 @@ pub fn handle_err(
     end: usize,
 )
 {
+    let num = err.len().div_ceil(get_terminal_dimensions().0) - 1;
     print!(
-        "\x1b[J\x1b[G\n{}{}\x1b[A\x1b[G\x1b[K{}{}{}",
+        "\x1b[J\x1b[G\n{}{}\x1b[G\x1b[A\x1b[K{}{}{}",
         err,
-        "\x1b[A".repeat(err.len().div_ceil(get_terminal_dimensions().0) - 1),
+        if num == 0
+        {
+            String::new()
+        }
+        else
+        {
+            format!("\x1b[{}A", num)
+        },
         prompt(options, colors),
         to_output(&input[start..end], options.color, colors),
         if options.color { "\x1b[0m" } else { "" },
