@@ -432,6 +432,7 @@ pub fn arg_opts(
                     args.remove(i);
                 }
             }
+            "--onaxis" => options.onaxis = !options.onaxis,
             "--comma" => options.comma = !options.comma,
             "--graph" => options.graph = !options.graph,
             "--sci" | "--scientific" => options.sci = !options.sci,
@@ -500,7 +501,7 @@ pub fn file_opts(
         let mut split;
         for line in reader.lines().map(|l| l.unwrap())
         {
-            if line.starts_with('#')
+            if line.starts_with('#') || line.is_empty()
             {
                 continue;
             }
@@ -617,6 +618,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "point" => format!("{}", options.point_style),
         "base" => format!("{}", options.base),
         "ticks" => format!("{}", options.ticks),
+        "onaxis" => format!("{}", options.onaxis),
         "decimal" | "deci" | "decimals" => format!("{}", options.decimal_places),
         "prec" | "precision" => format!("{}", options.prec.0),
         "graphprec" | "graphprecision" => format!("{}", options.graph_prec.0),
@@ -775,6 +777,7 @@ pub fn set_commands(
             "scalegraph" => options.scale_graph = true,
             "debug" => options.debug = true,
             "vars" => options.allow_vars = true,
+            "onaxis" => options.onaxis = true,
             _ => return Ok(()),
         }
     }
@@ -801,6 +804,7 @@ pub fn set_commands(
             "comma" => options.comma = false,
             "graph" => options.graph = false,
             "var_multiply" => options.var_multiply = false,
+            "onaxis" => options.onaxis = false,
             _ => return Ok(()),
         }
     }
@@ -997,7 +1001,6 @@ pub fn set_commands(
                                 }
                             }
                         }
-                        let mut redef = Vec::new();
                         for (i, var) in vars.to_vec().iter().enumerate()
                         {
                             if !var.unparsed.is_empty()
@@ -1013,7 +1016,6 @@ pub fn set_commands(
                                         func_vars.push((-1, i.iter().collect()));
                                     }
                                 }
-                                redef.push(var.name.clone());
                                 let mut fvs = Vec::new();
                                 let mut unparsed = var.unparsed.clone();
                                 if unparsed.contains(':')
@@ -1060,7 +1062,7 @@ pub fn set_commands(
                                     _ => return Err("prec crash"),
                                 };
                                 parsed.1.extend(fvs);
-                                vars[i].parsed = if l.contains('(')
+                                vars[i].parsed = if var.name.contains(&'(')
                                 {
                                     parsed.0
                                 }
@@ -1084,110 +1086,6 @@ pub fn set_commands(
                                 }
                             }
                         }
-                        let mut k = 0;
-                        while k < redef.len()
-                        {
-                            for (j, v) in vars.to_vec().iter().enumerate()
-                            {
-                                if redef[k] != v.name
-                                    && v.unparsed.contains(
-                                        &redef[k][0..=redef[k]
-                                            .iter()
-                                            .position(|a| a == &'(')
-                                            .unwrap_or(redef[k].len() - 1)],
-                                    )
-                                {
-                                    let mut func_vars: Vec<(isize, String)> = Vec::new();
-                                    if v.name.contains(&'(')
-                                    {
-                                        let mut l = v.name.clone();
-                                        l.drain(0..=l.iter().position(|c| c == &'(').unwrap());
-                                        l.pop();
-                                        for i in l.split(|c| c == &',')
-                                        {
-                                            func_vars.push((-1, i.iter().collect()));
-                                        }
-                                    }
-                                    let mut fvs = Vec::new();
-                                    let mut unparsed = v.unparsed.clone();
-                                    if unparsed.contains(':')
-                                    {
-                                        let un = unparsed;
-                                        let mut split = un.split(':').collect::<Vec<&str>>();
-                                        unparsed = split.pop().unwrap().to_string();
-                                        for i in split
-                                        {
-                                            if i.contains('=')
-                                            {
-                                                let mut split = i.splitn(2, '=');
-                                                let s = split.next().unwrap().to_string();
-                                                let parsed = input_var(
-                                                    split.next().unwrap(),
-                                                    vars.to_vec(),
-                                                    &mut func_vars,
-                                                    &mut 0,
-                                                    *options,
-                                                    false,
-                                                    false,
-                                                    0,
-                                                    s.chars().collect::<Vec<char>>(),
-                                                )?;
-                                                func_vars.push((-1, s.clone()));
-                                                fvs.push((s, parsed.0));
-                                                fvs.extend(parsed.1)
-                                            }
-                                        }
-                                    }
-                                    let mut parsed = match input_var(
-                                        &unparsed,
-                                        vars.to_vec(),
-                                        &mut func_vars,
-                                        &mut 0,
-                                        *options,
-                                        false,
-                                        false,
-                                        0,
-                                        v.name.clone(),
-                                    )
-                                    {
-                                        Ok(n) => (n.0, n.1),
-                                        _ => return Err("prec crash"),
-                                    };
-                                    parsed.1.extend(fvs);
-                                    if v.name.contains(&'(')
-                                    {
-                                        if vars[j].parsed.clone() != parsed.0
-                                            || vars[j].funcvars.clone() != parsed.1
-                                        {
-                                            redef.push(v.name.clone());
-                                            vars[j].parsed = parsed.0.clone();
-                                            vars[j].funcvars = parsed.1.clone();
-                                        }
-                                    }
-                                    else if let Ok(n) =
-                                        do_math(parsed.0.clone(), *options, parsed.1.clone())
-                                    {
-                                        if n != vars[j].parsed[0]
-                                        {
-                                            redef.push(v.name.clone());
-                                            vars[j].parsed = vec![n];
-                                        }
-                                    }
-                                    if v.name.contains(&'(')
-                                        && v.unparsed
-                                            .contains(v.name.split(|c| c == &'(').next().unwrap())
-                                        && (v.unparsed.contains("piecewise")
-                                            || v.unparsed.contains("pw"))
-                                    {
-                                        let parsed = vars[j].parsed.clone();
-                                        vars[j]
-                                            .funcvars
-                                            .push((v.name.iter().collect::<String>(), parsed))
-                                    }
-                                }
-                            }
-                            k += 1;
-                        }
                     }
                 }
                 _ => return Err("Invalid precision"),
@@ -1981,6 +1879,7 @@ pub fn silent_commands(options: &mut Options, input: &[char])
         "color" => options.color = !options.color,
         "prompt" => options.prompt = !options.prompt,
         "depth" => options.depth = !options.depth,
+        "onaxis" => options.onaxis = !options.onaxis,
         "surface" => options.surface = !options.surface,
         "flat" => options.flat = !options.flat,
         "deg" => options.deg = Degrees,
@@ -2017,73 +1916,79 @@ pub fn commands(
     {
         "default" | "defaults" | "reset" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             *options = Options::default();
         }
         "var_multiply" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.var_multiply = !options.var_multiply
         }
         "scalegraph" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.scale_graph = !options.scale_graph
         }
         "color" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.color = !options.color;
         }
         "prompt" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.prompt = !options.prompt;
         }
         "depth" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.depth = !options.depth;
         }
+        "onaxis" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.onaxis = !options.onaxis;
+        }
         "surface" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.surface = !options.surface;
         }
         "flat" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.flat = !options.flat;
         }
         "deg" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.deg = Degrees;
         }
         "rad" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.deg = Radians;
         }
         "grad" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.deg = Gradians;
         }
         "rt" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.real_time_output = !options.real_time_output;
         }
@@ -2091,13 +1996,13 @@ pub fn commands(
         "pi" => options.tau = false,
         "small_e" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.small_e = !options.small_e;
         }
         "sci" | "scientific" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.sci = !options.sci;
         }
@@ -2109,61 +2014,61 @@ pub fn commands(
         }
         "debug" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.debug = !options.debug;
         }
         "help" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             help();
         }
         "line" | "lines" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.lines = !options.lines;
         }
         "polar" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.polar = !options.polar;
         }
         "frac" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.frac = !options.frac;
         }
         "multi" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.multi = !options.multi;
         }
         "tabbed" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.tabbed = !options.tabbed;
         }
         "comma" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.comma = !options.comma;
         }
         "graph" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.graph = !options.graph;
         }
         "history" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             for l in lines
             {
                 print!("{}\x1b[G\n", l);
@@ -2172,7 +2077,7 @@ pub fn commands(
         }
         "vars" | "variables" =>
         {
-            print!("\x1b[A\x1b[G\x1b[K");
+            print!("\x1b[G\x1b[A\x1b[K");
             for v in vars.iter()
             {
                 if !v.unparsed.is_empty()
@@ -2256,13 +2161,13 @@ pub fn commands(
             stdout.flush().unwrap();
         }
         "version" => println!(
-            "\x1b[A\x1b[G\x1b[K{} {}\x1b[G",
+            "\x1b[G\x1b[A\x1b[K{} {}\x1b[G",
             env!("CARGO_PKG_NAME"),
             env!("CARGO_PKG_VERSION")
         ),
         "exit" | "quit" | "break" =>
         {
-            print!("\x1b[A\x1b[G\x1b[J");
+            print!("\x1b[G\x1b[A\x1b[J");
             stdout.flush().unwrap();
             terminal::disable_raw_mode().unwrap();
             std::process::exit(0);
@@ -2274,7 +2179,7 @@ pub fn commands(
             let next = split.next().unwrap();
             if next == "history"
             {
-                print!("\x1b[A\x1b[G\x1b[K");
+                print!("\x1b[G\x1b[A\x1b[K");
                 let r = split.next().unwrap();
                 for i in lines
                 {
