@@ -644,10 +644,7 @@ pub fn input_var(
                             if a.real().is_sign_negative()
                             {
                                 output.pop();
-                                output.push(Num(Complex::with_val(
-                                    options.prec,
-                                    (-a.real(), a.imag()),
-                                )));
+                                output.push(Num(-a.clone()));
                                 output.insert(output.len() - 1, Num(n1.clone()));
                                 output.insert(output.len() - 1, Str("*".to_string()));
                             }
@@ -991,7 +988,7 @@ pub fn input_var(
             if wordv == a.1
             {
                 num = a.0;
-                word = a.1.clone();
+                word.clone_from(&a.1);
                 true
             }
             else
@@ -1005,7 +1002,7 @@ pub fn input_var(
                 if wordv.starts_with(&a.1) && !a.1.is_empty()
                 {
                     num = a.0;
-                    word = a.1.clone();
+                    word.clone_from(&a.1);
                     true
                 }
                 else
@@ -1214,7 +1211,7 @@ pub fn input_var(
                             let mut var = var;
                             for (varf, func_var) in split.iter().zip(func_vars)
                             {
-                                let num = if let Ok(n) =
+                                let mut num = if let Ok(n) =
                                     Complex::parse(varf.iter().collect::<String>())
                                 {
                                     vec![Num(n.complete(options.prec))]
@@ -1268,13 +1265,13 @@ pub fn input_var(
                                     {
                                         let iden =
                                             format!("@{}{}{}{}@", i, func_var, depth, vars.len());
-                                        funcvars.extend(func);
                                         if parsed.len() == 1
                                         {
                                             parsed
                                         }
                                         else
                                         {
+                                            funcvars.extend(func);
                                             funcvars.push((iden.clone(), parsed));
                                             vec![Str(iden)]
                                         }
@@ -1292,6 +1289,11 @@ pub fn input_var(
                                         }]
                                     }
                                 };
+                                if print && num.len() == 1
+                                {
+                                    num.insert(0, Str("(".to_string()));
+                                    num.push(Str(")".to_string()));
+                                }
                                 let mut k = 0;
                                 for (x, fv) in var.funcvars.clone().iter().enumerate()
                                 {
@@ -1450,7 +1452,8 @@ pub fn input_var(
                                 .collect::<String>();
                             let mut var = var;
                             let mut k = 0;
-                            let num = if let Ok(n) = Complex::parse(temp.iter().collect::<String>())
+                            let mut num = if let Ok(n) =
+                                Complex::parse(temp.iter().collect::<String>())
                             {
                                 vec![Num(n.complete(options.prec))]
                             }
@@ -1502,13 +1505,13 @@ pub fn input_var(
                                     })
                                 {
                                     let iden = format!("@{}{}{}{}@", i, l, depth, vars.len());
-                                    funcvars.extend(func);
                                     if parsed.len() == 1
                                     {
                                         parsed
                                     }
                                     else
                                     {
+                                        funcvars.extend(func);
                                         funcvars.push((iden.clone(), parsed));
                                         vec![Str(iden)]
                                     }
@@ -1526,6 +1529,11 @@ pub fn input_var(
                                     }]
                                 }
                             };
+                            if print && num.len() == 1
+                            {
+                                num.insert(0, Str("(".to_string()));
+                                num.push(Str(")".to_string()));
+                            }
                             while k < var.parsed.len()
                             {
                                 if var.parsed[k].str_is(&l)
@@ -1737,9 +1745,9 @@ pub fn input_var(
                         }
                         if print
                         {
-                            output.push(Str(var.name.iter().collect::<String>()));
+                            output.push(Str("(".to_string()));
                         }
-                        else if !var.parsed.is_empty()
+                        if !var.parsed.is_empty()
                         {
                             output.push(var.parsed[0].clone());
                         }
@@ -1747,6 +1755,10 @@ pub fn input_var(
                         {
                             return Err("bad input2");
                         }
+                        if print
+                        {
+                            output.push(Str(")".to_string()));
+                        }
                         if scientific
                         {
                             output.push(Str(")".to_string()));
